import sys
import re
import click
import datetime
from dateutil.relativedelta import relativedelta
import rvo.db as db
import rvo.utils as utils
import rvo.views as views
from rvo.cli import validate_date
import rvo.analysis as analysis
from rvo.crypto import crypto
from rvo.validate import validate
from tabulate import tabulate
import rvo.config

@click.command(short_help="Text analysis on a document",
               help="""
               To be honest, I just wanted to play with nltk.
               Also i like statistics and this kind of stuff.

               So this will print out some natrual language facts about
               the document given.
               """)
@click.option('password', '-p', '--password', required=False, default=False,
              help="Password for encrypted documents")
@click.option('-c', '--category', type=str, multiple=True,
              help='filter for category')
@click.option('-t', '--tag', type=str, multiple=True,
              help='filter for tags')
@click.option('objectid', '-i', '--id', type=str, default=False,
              help='only on a single document by id')
@click.pass_context
def stats(ctx, objectid, category, tag, password):
    """
    :docid: str
    :returns: bool
    """

    coll = db.get_document_collection(ctx)

    if objectid:
        docs = []
        doc, docid = db.get_document_by_id(ctx, str(objectid))
        docs.append(doc)
    else:
        tags = utils.normalize_element(tag, "tags")
        categories = utils.normalize_element(category, "categories")
        query = {"$and": [ tags, categories ] }
        coll = db.get_document_collection(ctx)

        config = ctx.obj["config"]

        docs = coll.find(query).sort("updated", -1)

    content = ""
    tags = []
    categories = []
    c = False
    for doc in docs:
        if c is False:
            doc["content"],c = db.get_content(ctx, doc, crypto_object=c, password=password)

        content += doc["content"]
        tags.extend(doc["tags"])
        categories.extend(doc["categories"])

    if len(content) == 0:
        utils.log_error("No documents found with this query")
        return False


    utils.log_info("Text analysis\n")
    table = []
    headers = ["Analysis", "Result"]

    try:
        analysis.get_sentences("this is a test")
    except LookupError:
        utils.log_error("NLTK is needed to do text analysis on your document")
        utils.log_error("In order to do this, execute:")
        utils.log_error(' python -c \'import nltk; nltk.download("book")\'')
        sys.exit(1)

    table.append(["Sentences" , analysis.get_sentences(content)])
    table.append(["Words", analysis.get_words(content)])
    table.append(["Characters", analysis.get_characters(content)])
    table.append(["Tags", len(tags)])
    table.append(["Categories", len(categories)])
    table.append(["Size of documents", analysis.get_size_of_document(content)])
    if objectid:
        table.append(["Age of document", analysis.get_age_of_document(doc["created"])])

    cwords = ""
    fdist = analysis.get_word_distribution(content)

    for w, f in analysis.get_most_common_words(fdist, 5):
        cwords = cwords + w + "(" + str(f) + ") "
    table.append(["Most common words", cwords])

    lwords = ""
    for f in analysis.get_least_common_words(fdist, 5):
       lwords = lwords + " " + f
    table.append(["Least common words", lwords])
    table.append(["Words per sentence", analysis.get_words_per_sentence(content)])

    longwords = ""
    for f in analysis.get_long_words(fdist, 5):
       longwords = longwords + " " + f
    table.append(["Long words", longwords])

    print tabulate(table, headers=headers)
    print("")

    return True



"""
crypto is used for encryption and decryption
the documents.

init_master and get_master are there for an additional
layer (indirection). Your password only encrypts the
master key once.

Every document is later encrypted and decrypted by the master
key that was generated once.
"""
import sys
import nacl.secret
import nacl.utils
import click
from pyblake2 import blake2b
import utils
import rvo.db as db

class crypto(object):
    """ The crypto class for everything rvo needs. """

    def __init__(self, ctx, password=False):
        self.collection = db.get_config_collection(ctx)
        self.ctx = ctx

        r = self.collection.find_one({"masterkey": {"$exists": True}})
        if r is None:
            self.init_master()

        self._masterkey = self.get_master(password)


    def init_master(self):
        """
        Generates the master key to encrypt all documents later
        Will ask for password to initialize and will put into the collection config
        :returns: bool
        """

        # Configuration collection
        coll = self.collection

        # generate random 256 byte to use as plain masterkey
        masterkey = blake2b(digest_size=32)
        masterkey.update(nacl.utils.random(256))
        masterkey = masterkey.digest()

        # ask for password to de/encrypt the masterkey
        key = blake2b(digest_size=32)
        utils.log_info("Encryption has not been initialized yet. Please enter your password.")
        pw = click.prompt("%s Set Password" % utils.query_prefix, type=str, hide_input=True, confirmation_prompt=True)
        key.update(pw.encode("utf-8"))
        key = key.digest()
        utils.log_info("Password set successfully. Dont forget it, otherwise you are fucked.")

        # encrypt masterkey with user password
        box = nacl.secret.SecretBox(key)
        nonce = nacl.utils.random(nacl.secret.SecretBox.NONCE_SIZE)
        enc = box.encrypt(masterkey, nonce, encoder=nacl.encoding.HexEncoder)

        r = coll.insert({"masterkey": enc})
        return True

    def get_master(self, password):
        """
        Will be triggered everytime the user tries to access an encrypted
        document from the database. Asks for his password, decrypts master key
        and returns the masterkey what will be an attribute from the crypto class
        :returns: str
        """
        # Fetch encrypted masterkey from db
        coll = self.collection
        masterkey = coll.find_one({"masterkey": {"$exists": True}})
        masterkey = masterkey["masterkey"]

        # hash input pw
        key = blake2b(digest_size=32)

        if password is False:
            password = click.prompt("%s Password" % utils.query_prefix, type=str, hide_input=True)

        key.update(password.encode("utf-8"))
        key = key.digest()


        # init box
        box = nacl.secret.SecretBox(key)

        # use password to decrypt masterkey
        try:
            masterkey = box.decrypt(ciphertext=masterkey, encoder=nacl.encoding.HexEncoder)
            return masterkey
        except nacl.exceptions.CryptoError:
            utils.log_error("Invalid Password")
            sys.exit(1)
            return False


    def encrypt_content(self, content):
        """
        Will encrypt the content that is given with the masterkey from crypto class
        :content: str (plaintext)
        :returns: str (encrypted and encoded in hexdigest)
        """

        # init new blake2b keystore
        # and get password as hex
        try:
            key = blake2b(digest_size=32)
            key.update(self._masterkey)
            key = key.digest()
        except TypeError:
            sys.exit(1)
        box = nacl.secret.SecretBox(key)

        # generate a nonce
        nonce = nacl.utils.random(nacl.secret.SecretBox.NONCE_SIZE)

        try:
            msg = box.encrypt(content, nonce, encoder=nacl.encoding.HexEncoder)
            return msg
        except nacl.exceptions.CryptoError:
            utils.log_error("Invalid Password")
            return False

    def decrypt_content(self, content):
        """
        Will decrypt the content that is given as argument using the
        masterkey from crypto class.
        :content: str (encrypted and encoded in hexdigest)
        :returns: str (plaintext)
        """

        # init new blake2b keystore and hash
        # masterkey
        try:
            key = blake2b(digest_size=32)
            key.update(self._masterkey)
            key = key.digest()
        except TypeError:
            sys.exit(1)

        box = nacl.secret.SecretBox(key)

        content = content.encode("utf-8")

        try:
            plain = box.decrypt(ciphertext=content, encoder=nacl.encoding.HexEncoder)
            return plain
        except nacl.exceptions.CryptoError:
            utils.log_error("Invalid Password")
            return False



import sys
import click
import pprint
from bson import ObjectId
from bson.json_util import dumps
import rvo.db as db
import rvo.utils as utils
import rvo.transaction as transaction
from rvo.crypto import crypto
import rvo.config

@click.command(short_help="Exports all document",
              help="""
              Exports documents.

              In case you want to export some of your documents
              to another piece of code, or build up a website out of
              some of the contents, you can export documents.

              You are free to choose the output format.
              You can also filter on specific documents to export on category
              or on tag.
              """)
@click.option('format', '--to', required=False, default="json",
              type=click.Choice(['json', 'markdown']),
              help="Specify the format of the exported output")
@click.option('password', '-p', '--password', required=False, default=False,
              help="Password for encrypted documents")
@click.option('-c', '--category', type=str, multiple=True,
              help='Exports need to be in this category')
@click.option('-t', '--tag', type=str, multiple=True,
              help='Exports have to contain this tag')
@click.option('objectid', '-i', '--id', type=str, default=False,
              help='Exports document by id')
@click.pass_context
def export(ctx, format, password, category, tag, objectid):
    """
    Shows a single object from database
    and opens its content in a pager
    :docid: string (will be converted to bson object)
    :returns: bool
    """

    if objectid:
        docs = []
        doc, docid = db.get_document_by_id(ctx, str(objectid))
        docs.append(doc)
    else:

        tags = utils.normalize_element(tag, "tags")
        categories = utils.normalize_element(category, "categories")

        query = {"$and": [ tags, categories ] }

        coll = db.get_document_collection(ctx)

        config = ctx.obj["config"]

        docs = coll.find(query).sort("updated", -1)

    if format == "json":
        export_json(ctx, docs, password)
    if format == "markdown":
        export_markdown(ctx, docs, password)

    return True

def export_json(ctx, docs, password):
    jsondata = []
    for doc in docs:
        if doc["encrypted"] is True and password is not False:
            doc["content"], c = db.get_content(ctx, doc, password=password)
        jsondata.append(doc)

    print(dumps(jsondata))

def export_markdown(ctx, docs, password):
    for doc in docs:
        if doc["encrypted"] is True and password is not False:
            doc["content"], c = db.get_content(ctx, doc, password=password)
        else:
            doc["content"] = doc["content"].encode('utf8')

        print doc["content"]



import sys
import click
import email
import quopri
import pprint
import datetime
from bson import ObjectId
from bson.json_util import dumps
import rvo.db as db
import rvo.utils as utils
import rvo.transaction as transaction
from email.header import decode_header
from rvo.crypto import crypto
from rvo.validate import validate
import rvo.config

@click.command('import', short_help="Import documents",
              help="""
              Import documents.
              """)
@click.option('format', '--from', required=False, default="json",
              type=click.Choice(['json', 'mail']),
              help="Specify the format of the input")
@click.option('-c', '--category', type=str, multiple=True,
              help='Import to this category')
@click.option('-t', '--tag', type=str, multiple=True,
              help='Import with this tag')
@click.pass_context
def rimport(ctx, format, category, tag):
    """ Import to rvo
    :returns: bool
    """

    if format == "json":
        import_json()
    if format == "mail":
        import_mail(ctx, tag, category)

    return True

def import_json():
    print("JSON not implemented yet")

def import_mail(ctx, tag, category):
    content = ""
    for l in click.get_text_stream('stdin'):
        content = content + l
    msg = email.message_from_string(content)

    # title
    subject, encoding = email.header.decode_header(msg['Subject'])[0]
    if encoding is None:
        encoding = "utf-8"

    title = subject.decode(encoding)

    # content
    content = msg.get_payload(decode=False)
    content = quopri.decodestring(content)
    content = "# " + title + '\n\n' + content
    date = datetime.datetime.now()

    coll = db.get_document_collection(ctx)
    config = ctx.obj["config"]

    item = {
        "title": title,
        "content": content,
        "tags": list(tag),
        "categories": list(category),
        "created": date,
        "updated": date,
        "encrypted": False,
    }

    # insert item if its valid
    if validate(item):
        coll = db.get_document_collection(ctx)
        docid = coll.insert_one(item).inserted_id

        transaction.log(ctx, str(docid), "import", title)
        utils.log_info("Document \"%s\" created." % title)

    else:
        utils.log_error("Validation of the updated object did not succeed")



import datetime
from geopy.geocoders import Nominatim
from dateutil.parser import parse

# validators
def validate_date(ctx, param, value):
    try:
        value = parse(value)
        return value
    except (AttributeError, TypeError) as e:
        return value
    except ValueError:
        raise click.BadParameter('Date format \"%s\" not valid' % value)

def validate_location(ctx, param, value):

        if value is None:
            return value

        geolocator = Nominatim()
        location = geolocator.geocode(value)

        if location is None:
            raise click.BadParameter('Location \"%s\" could not be found' % value)

        return location

def validate(document):

    fields = [
        "title",
        "encrypted",
        "content",
        "created",
        "updated",
        "tags",
        "categories"
    ]

    # Check if we have an document
    if not isinstance(document, dict):
        return False

    # Verifiy if all fields are existent
    try:
        for field in fields:
            document[field]
    except KeyError:
        return False

    # Check if content is utf8
    if not isinstance(document["content"], str):
        if not isinstance(document["content"], unicode):
            return False

    # # Check title utf8
    if not isinstance(document["title"], str):
        if not isinstance(document["title"], unicode):
            return False

    # Check if tags is a list
    if not isinstance(document["tags"], list):
        return False

    # Check if categories is a list
    if not isinstance(document["categories"], list):
        return False

    # Check if create is a valid datetime object
    if not isinstance(document["created"], datetime.datetime):
        return False

    # Check if updated is a valid datetime object
    if not isinstance(document["updated"], datetime.datetime):
        return False

    if not isinstance(document["encrypted"], bool):
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import sys
import nltk
import datetime
import hurry.filesize

def get_sentences(content):
    """
    Returns the number of sentences
    :content: str
    :returns: int
    """
    sentences = len(nltk.tokenize.sent_tokenize(content))
    return sentences

def get_words(content):
    """
    Returns the number of sentences
    :content: str
    :returns: int
    """
    words = len(nltk.word_tokenize(content))
    return words

def get_characters(content):
    """
    Returns the number of sentences
    :content: str
    :returns: int
    """
    characters = len(content)
    return characters

def get_word_distribution(content):
    """
    Returns a distribution file on words
    """

    # remove urls
    content = re.sub(r'^https?:\/\/.*[\r\n]*', '', content, flags=re.MULTILINE)

    # Remove non alphanumeric chars
    tokenizer = nltk.tokenize.RegexpTokenizer(r'\w+')
    words = tokenizer.tokenize(content)

    # Remove single-character tokens (mostly punctuation)
    words = [word for word in words if len(word) > 1]

    # Remove numbers
    words = [word for word in words if not word.isnumeric()]

    # Lowercase all words (default_stopwords are lowercase too)
    words = [word.lower() for word in words]

    # Remove numbers
    words = [word for word in words if not word.isnumeric()]

    # Remove english stopwords
    words = [word for word in words if word not in nltk.corpus.stopwords.words('english')]

    # Remove german stopwords
    words = [word for word in words if word not in nltk.corpus.stopwords.words('german')]


    # Calculate frequency distribution
    fdist = nltk.FreqDist(words)

    return fdist


def get_most_common_words(fdist, count):
    """
    Get list of words that are most commonly
    used in this text
    :fdist: nltk object
    :words: int - number of words as result
    :returns: tuple - (words,frequency)
    """
    return fdist.most_common(count)

def get_least_common_words(fdist, count):
    """
    Get list of words that are least commonly
    used in this text
    :fdist: nltk object
    :count: int - number of words as result
    :returns: list
    """
    return fdist.hapaxes()[0:count]

def get_words_per_sentence(content):
    """
    Get words per sentance average
    :content: str
    :returns: int
    """
    words = get_words(content)
    sentences = get_sentences(content)
    return words / sentences

def get_long_words(fdist,count):
    """
    Get list of words that are least commonly
    used in this text
    :content: str
    :count: int - number of words as result
    :returns: list
    """
    longwords = []
    for word in fdist:
        if len(word) > 10 and len(word) < 70:
            longwords.append(word)

    longwords = sorted(longwords, key=len, reverse=True)

    return longwords[0:count]

def get_age_of_document(created):
    """
    Returns the age of a document as
    string.
    :created: date
    :returns: str
    """
    today = datetime.datetime.now()
    age = today - created
    return str(age)

def get_size_of_document(content):
    """
    Returns human readable size of document
    :content: str
    :returns: str
    """
    s = sys.getsizeof(content)
    s = hurry.filesize.size(s, system=hurry.filesize.alternative)

    return s



import sys
import pymongo
import bson
from bson.objectid import ObjectId
import rvo.utils as utils
import rvo.config

def db_ping(ctx):
    """ Ping the connection to mongodb database
    :returns: bool
    """
    try:
        config = ctx.obj["config"]
        c = ctx.obj["db"]
        c.server_info()
        return True
    except:
        return False

def db_verify(ctx):
    """ Does a test write, read and remove
    :returns: bool
    """
    try:
        # Define name
        testcoll = "test_collection"

        # Fetch config and get db object
        config = ctx.obj["config"]
        c = ctx.obj["db"]
        db = c[config["db"]]

        # Get collection object
        coll = get_collection(ctx, testcoll)

        # Do write test
        coll.insert({"a": 1})

        # Do read test
        resp = coll.find_one({})
        if resp == {"a": 1}:
            pass

        # Do removal test
        coll.remove({})

        # Drop test_collection
        db.drop_collection(testcoll)

        return True
    except:
        return False

def get_collection(ctx, coll):
    """ Initialize MongoDB Connection
    :returns: collection object
    """
    config = ctx.obj["config"]
    c = ctx.obj["db"]
    db = c[config["db"]]
    collection = db[coll]
    return collection

def get_document_collection(ctx):
    """
    Get document collection handle
    from config.
    :returns: collection object
    """
    config = ctx.obj["config"]
    collection = get_collection(ctx, config["collection"])
    return collection

def get_transactions_collection(ctx):
    """
    Get document collection handle
    from config.
    :returns: collection object
    """
    config = ctx.obj["config"]
    collection = get_collection(ctx, config["transactions"])
    return collection

def get_config_collection(ctx):
    """
    Get document collection handle
    from config.
    :returns: collection object
    """
    config = ctx.obj["config"]
    collection = get_collection(ctx, config["config"])
    return collection

def get_shortids_collection(ctx):
    """
    Get document collection handle
    from config.
    :returns: collection object
    """
    config = ctx.obj["config"]
    collection = get_collection(ctx, config["shortids"])
    return collection

def check_for_duplicate(ctx, field, content):
    """ Check if url already exists in db
    :returns: bool
    """
    config = ctx.obj["config"]
    coll = get_collection(ctx, config["collection"])

    if coll.find({field: content}).count() > 0:
        return True
    else:
        return False

def get_document_by_id(ctx, id):
    """
    Gets a document by its id, exits if false
    :coll: pymongo collection object
    :id: str
    :returns: dict, docid
    """
    coll = get_document_collection(ctx)
    shortids = get_shortids_collection(ctx)

    try:
        doc = shortids.find_one({"sid": int(id)})
        id = str(doc["oid"])
    except (IndexError, ValueError, TypeError) as e:
        pass

    try:
        doc = coll.find_one({"_id": ObjectId(id)})
        if doc is None:
            utils.log_info("No Results for %s" % id)
            sys.exit(1)
        return doc, id
    except bson.errors.InvalidId:
        utils.log_error("Error: %s is not a valid ID or ObjectId." % id)
        sys.exit(1)

def map_shortid(ctx, sid, oid):
    """
    Maps oid to a short id for better usage
    in commandline. At the end you have to type
    "edit 2" and not "edit 56823D28ds23821"
    Mapping will be stored in a separate collection
    and should be resettet with each view.
    :coll: pymongo collection object
    :sid: str (bson object id)
    :oid: str (bson object id)
    :returns: dict
    """
    shortids = get_shortids_collection(ctx)
    shortids.insert({"sid": sid, "oid": oid})
    return True

def clean_shortids(ctx):
    """
    Cleans all shortids from cache collection.
    Necessary to get fresh results for every search
    :coll: pymongo collection object
    :returns: bool
    """
    shortids = get_shortids_collection(ctx)
    shortids.remove({"$and": [{"sid": {"$exists": True}}, {"oid": {"$exists": True}}]})
    return True

def add_transaction(ctx, item):
    transactions = get_transactions_collection(ctx)
    transactions.insert(item)
    return True

def get_content(ctx, doc, crypto_object=False, password=False):
    """
    Get content of a document.
    For later use, it also returns the initialized crypto object
    :doc: document object
    :crypto_object: instance of class rvo.crypto.crypto
    returns: str, crypto_object
    """
    from rvo.crypto import crypto
    c = crypto_object
    if doc["encrypted"] is True:
        if c is False:
            c = crypto(ctx=ctx, password=password)
        content = c.decrypt_content(doc["content"])
        content = content.decode("utf8")
    else:
        content = doc["content"]

    return content, c



import re
import sys
import os
import tempfile
import subprocess
import urllib2
import ssl
import click
from bs4 import BeautifulSoup

# Content utils

def get_content_from_editor(config, template=""):
    """
    Writes content to file and opens up the editor that is
    parameter config. Returns new content and deletes file
    :config: str (commands)
    :template: str
    :returns: bool
    """
    _, tmpfile = tempfile.mkstemp(prefix="rvo-", text=True, suffix=".md")
    with open(tmpfile, 'w') as f:
        if template:
            try:
                f.write(template.encode("utf-8"))
            except (UnicodeEncodeError, UnicodeDecodeError) as e:
                f.write(template)
    subprocess.call(config.split() + [tmpfile])
    with open(tmpfile, "r") as f:
        raw = f.read()
    if not raw:
        log_error("Error: No content from editor")
        sys.exit(1)
    return raw, tmpfile

def view_content_in_pager(config, opts, template=""):
    """
    Writes content to file and opens up the pager that is
    parameter config. Temp file will be deleted afterwards.
    :config: str (commands)
    :opts: str (pager options for commandline)
    :template: str
    :returns: bool
    """
    _, tmpfile = tempfile.mkstemp(prefix="rvo-", text=True, suffix=".md")
    with open(tmpfile, 'w') as f:
        if template:
            try:
                f.write(template.encode("utf-8"))
            except (UnicodeEncodeError, UnicodeDecodeError) as e:
                f.write(template)

    if opts is not None:
        subprocess.call(config.split() + [opts] + [tmpfile])
    else:
        subprocess.call(config.split() + [tmpfile])

    os.remove(tmpfile)
    return True

def get_title_from_webpage(url):
    """ Fetch <title> of a html site for title element
    :url: str (http url)
    :returns: str
    """

    # LOL SECURITY
    ssl._create_default_https_context = ssl._create_unverified_context

    try:
        h = {'User-Agent':'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_2) AppleWebKit/601.3.9 (KHTML, like Gecko) Version/9.0.2 Safari/601.3.9'}
        u = urllib2.Request(url, headers=h)
        u = urllib2.urlopen(u)
        soup = BeautifulSoup(u, "html.parser")
        s = soup.title.string.replace('\n', ' ').replace('\r', '').lstrip().rstrip()
        s = s.lstrip()
        return s
    except (AttributeError, MemoryError, ssl.CertificateError, IOError) as e:
        return "No title"
    except ValueError:
        return False

def get_title_from_content(content):
    """
    Generates a title from the content
    using the first line and stripping away whitespace
    and hash signs
    :content: str
    :returns: str

    """
    title = content.split('\n', 1)[0].replace('#', '').lstrip()[0:50]
    return title

def clean_tmpfile(tmpfile):
    """
    Basically, this removes a file. Its used to only
    delete the file that the editor leaves behind in case that
    rvo crashes.
    """
    try:
        os.remove(tmpfile)
    except OSError:
        pass
    return True

# Styling

query_prefix = '\033[38;5;154m>\033[38;5;118m>\033[38;5;120m>\033[0m'

def log_error(msg):
    """
    Error output on terminal
    """
    prefix = '\033[38;5;196m>\033[38;5;202m>\033[38;5;208m>\033[0m'
    click.echo(prefix + " " + msg)

def log_info(msg):
    """
    Info output on terminal
    """
    prefix = '\033[38;5;129m>\033[38;5;135m>\033[38;5;141m>\033[0m'
    click.echo(prefix + " " + msg)

# OS and Testing Utils

def isatty():
    """
    Wrapper for isatty
    I use this for monkeypatching input detection
    in the add method. Its not nice. It works. Its mine <3
    """
    return sys.stdin.isatty()

# Transformation of elements

def normalize_element(filter, field):
    """
    Turn items into a dict and apply regexes
    """

    if len(filter) > 0:
        filters = []
        for f in filter:
            filters.append(re.compile(f, re.IGNORECASE))
        filters = {field: { "$in": filters}}
    else:
        filters = {}

    return filters

def remove_emojis(content):
    """
	Removes emojis from a string
	:content: str
    :returns: str
    """
    content = unicode(content)
    try:
        emojis = re.compile(u'['
        u'\U0001F300-\U0001F64F'
        u'\U0001F680-\U0001F6FF'
        u'\u2600-\u26FF\u2700-\u27BF]+',
        re.UNICODE)
    except re.error:
        emojis = re.compile(u'('
        u'\ud83c[\udf00-\udfff]|'
        u'\ud83d[\udc00-\ude4f\ude80-\udeff]|'
        u'[\u2600-\u26FF\u2700-\u27BF])+',
        re.UNICODE)

    return emojis.sub('', content) # no emoji



import os
import ConfigParser
import rvo.utils as utils

def parse_config(conf="~/.rvo.conf"):

    # soon....
    default = {}
    con = dict()
    config = ConfigParser.ConfigParser()

    if not config.read(os.path.expanduser(conf)):
        utils.log_error("Error: your con %s could not be read" % conf)
        exit(1)

    try:
        con["uri"] = config.get("General", "MongoDB")
    except ConfigParser.NoOptionError:
        utils.log_error("Error: Please set a MongoDB Connection URI in %s" % conf)
        exit(1)

    try:
        con["db"] = config.get("General", "DB")
    except ConfigParser.NoOptionError:
        con["db"] = "rvo"

    try:
        con["editor"] = config.get("General", "Editor")
    except ConfigParser.NoOptionError:
        con["editor"] = "vim"

    try:
        con["pager"] = config.get("General", "Pager")
    except ConfigParser.NoOptionError:
        con["pager"] = "less"

    try:
        con["pageropts"] = config.get("General", "PagerOptions")
    except ConfigParser.NoOptionError:
        con["pageropts"] = None

    try:
        con["mailfrom"] = config.get("General", "MailFrom")
    except ConfigParser.NoOptionError:
        con["mailfrom"] = "nobody@example.net"

    con["collection"] = "documents"
    con["shortids"] = "cache"
    con["transactions"] = "transactions"
    con["config"] = "config"

    return con




import click
import rvo.db as db
import rvo.views as views

@click.command(short_help="Show transactions",
               help="""
               Transactions are logged informations
               about changes and access to the documents.

               `log' is used to get those transactions listed.

               Having a hard time remembering what you did?
               """)
@click.option('entries', '-e', '--entries', default=15, type=int,
              help='Number of entries being shown')
@click.pass_context
def log(ctx, entries):
    """
    Shows n latest transactions
    :n: int
    :returns: bool
    """
    coll = db.get_transactions_collection(ctx)

    SUM = {}
    c = 0
    print("")
    for doc in coll.find({}).sort("date", -1).limit(entries):
        c += 1
        SUM[c] = doc

    views.transactions(SUM, c+1)

    return True




#!/usr/bin/env python2.7
"""
Views are the rendering part. A dict object (most likly from mongodb)
will be displayed in a human readable, colorized way
"""
from tabulate import tabulate

def colorize(ty, msg):
    """
    Will colorize a given message using
    a color scheme (prompt) of a given type.
    :ty: str (type of colorizations)
    :msg: str (content to colorize)
    :returns: str (full colorized)
    """

    reset = '\033[0m'
    if ty == "title":
       pre = '\033[38;5;118m'
    if ty == 'tags':
       pre = '\033[38;5;97m'
    if ty == 'cats':
       pre = '\033[38;5;121m'
    if ty == 'header':
       pre = '\033[38;5;15m'

    return pre + msg + reset

def display_tags(tags):
    """
    Display tags if they are available
    :tags: list
    :returns: bool
    """
    print("tags:"),
    for e in tags:
        print(colorize(msg=e,ty="tags")),
    return True

def display_categories(cat):
    """
    Display categories if they are available
    :cat: list
    :returns: bool
    """
    print(" cats:"),
    for e in cat:
        print(colorize(msg=e,ty="cats")),
    return True

def table(documents, size):
    # print documents
    table = []
    headers = [
        colorize(msg="ID", ty="header"),
        colorize(msg="Title", ty="header"),
        colorize(msg="Cats", ty="header"),
        colorize(msg="Tags", ty="header"),
        colorize(msg="Date", ty="header")
    ]

    for x in reversed(range(1,size)):

        y = [
            documents[x]["sid"],
            colorize(msg=documents[x]["title"][0:60], ty="title"),
            colorize(msg=' '.join(documents[x]["categories"]), ty="cats"),
            colorize(msg=' '.join(documents[x]["tags"]),ty="tags"),
            documents[x]["created"].strftime("%Y-%m-%d")
        ]
        table.append(y)

    print tabulate(table, headers=headers)
    print ""

def detail(doc):
    """
    Display a single object using eventually meta
    wrappers from above with all meta data that exist.
    :r: dict (document from database)
    :returns: bool
    """

    # ID
    print("")
    print("   id: " + str(doc["_id"]))

    # Title
    print("title:"),
    print(colorize(msg=doc["title"],ty="title"))
    print(""),

    # Tags
    try:
        display_tags(doc["tags"])
    except KeyError:
        pass
    print("")

    # Categories
    try:
        display_categories(doc["categories"])
    except KeyError:
        pass
    print("")

    # Date
    print(" date: " + doc["created"].strftime('%Y-%m-%d %H:%M'))
    print("  mod: " + doc["updated"].strftime('%Y-%m-%d %H:%M'))
    print("  enc: " + str(doc["encrypted"]))

    # URL
    if any("links" in s for s in doc["categories"]):
        print("  url: " + doc["content"])

    print("")

    return True

def transactions(documents, size):
    """
    Display a single object using eventually meta
    wrappers from above
    :r: dict (document from database)
    :returns: bool
    """

    # print documents
    table = []
    headers = [
        colorize(msg="Date", ty="header"),
        colorize(msg="Type", ty="header"),
        colorize(msg="Title", ty="header"),
        colorize(msg="ObjectId", ty="header"),
    ]

    for x in reversed(range(1,size)):

        y = [
            documents[x]["date"].strftime("%Y-%m-%d %H:%M"),
            colorize(msg=documents[x]["type"], ty="cats"),
            colorize(msg=documents[x]["title"], ty="title"),
            str(documents[x]["_id"]),
        ]
        table.append(y)

    print tabulate(table, headers=headers)
    print ""

    return True



#!/usr/bin/env python2.7

import sys
import pymongo
import os
import click
import datetime
import rvo.utils as utils
from rvo import __version__
import rvo.config

command_folder = os.path.join(os.path.dirname(__file__), 'commands')
CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

# rvo command class
class rvoCommands(click.MultiCommand):

    def list_commands(self, ctx):
        rv = []
        for filename in os.listdir(command_folder):
            #if filename.endswith('.py'):
            if filename.endswith('.py') and not filename.startswith('__init__'):
                rv.append(filename[:-3])
        rv.sort()
        return rv

    def get_command(self, ctx, name):
        ns = {}
        fn = os.path.join(command_folder, name + '.py')
        try:
            with open(fn) as f:
                code = compile(f.read(), fn, 'exec')
                eval(code, ns, ns)
            return ns[name]
        except IOError:
            click.help_option()

# base help message
@click.command(cls=rvoCommands, context_settings=CONTEXT_SETTINGS,
help="""
Manage text data on commandline

\b
888,8, Y8b Y888P  e88 88e
888 "   Y8b Y8P  d888 888b
888      Y8b "   Y888 888P
888       Y8P     "88 88"

For the sake of your own data being managed
by you and only you!

""")
@click.version_option(version=__version__, prog_name="rvo")
@click.pass_context
def cli(ctx):
    ctx.obj = {}
    ctx.obj['config'] = rvo.config.parse_config()
    ctx.obj['db'] = pymongo.MongoClient(ctx.obj["config"]["uri"])

if __name__ == '__main__':
    cli()




import click
from bson import ObjectId
import rvo.db as db
import rvo.views as views
import rvo.utils as utils
import rvo.transaction as transaction

@click.command(help="""
               Deletes a document.

               Be careful. Delete means, its gone. Gone gone.
               There is no trashbin whatever. Its like burning a
               piece of paper. Maybe I should configure an alias to 'burn'.
               """)
@click.argument('docid')
@click.option('yes', '-y' ,'--yes', default=False, is_flag=True, help='Dont ask for confirmation')
@click.pass_context
def delete(ctx, docid, yes):
    """
    Deletes a document from the documentstore
    :docid: str (will be converted in bson object)
    :returns: bool
    """
    doc, docid = db.get_document_by_id(ctx, docid)
    views.detail(doc)
    coll = db.get_document_collection(ctx)

    t = doc["title"]
    if yes:
        coll.remove({"_id": ObjectId(docid)})
        utils.log_info("Removed %s" % t)
    else:
        if click.confirm("%s Are you sure, you want to delete this?" % utils.query_prefix):
            coll.remove({"_id": ObjectId(docid)})
            utils.log_info("Removed %s" % t)
            transaction.log(ctx, docid, "delete", t)
            return True




import re
import click
import datetime
from dateutil.relativedelta import relativedelta
import rvo.db as db
import rvo.utils as utils
import rvo.views as views
from rvo.validate import validate_date

@click.command(short_help="Some nostalgia by showing documents some years ago",
               help="""
               The memories command shows documents that were
               created exactly one (or more..) years ago today.

               \b
               Hint: This can be used to send an email to you for
               some entertainment.

               """)
@click.option('format', '--format', '-f',
              type=click.Choice(['table', 'detail']), default='table',
              help='Results by date ending at date')
@click.option('date', '-d', '--date', default=datetime.datetime.now(),
              help='Set a custom creation date for document', callback=validate_date)
@click.pass_context
def memories(ctx, format, date):
    """
    :format: str
    :returns: bool
    """
    coll = db.get_document_collection(ctx)

    documents = {}

    limit = 0
    utils.log_info("Documents having birthday today.\n")
    for year in range(1,40):
        start = date - relativedelta(years=year)
        start = start.replace(hour=0, minute=0, second=0, microsecond=0)
        end = date - relativedelta(years=year,days=-1)
        end = end.replace(hour=0, minute=0, second=0, microsecond=0)

        query = {"$and": [ {"created": {'$gte': start, '$lt': end}} ]}

        if format == "table":
            docs = coll.find(query)

            for doc in docs:
                limit += 1
                documents[limit] = doc


        if format == "detail":
            docs = coll.find(query)
            for doc in docs:
                views.detail(doc)

    # If not detail view - print all memories in one table
    if format == "table":
        db.clean_shortids(ctx)
        for x in reversed(range(1, limit+1)):
            db.map_shortid(ctx, sid=x, oid=documents[x]["_id"])
            documents[x]["sid"] = x

        views.table(documents, limit+1)


    return True



import click
import rvo.db as db
import rvo.utils as utils

@click.command(short_help="Verifys the connection to the database",
               help="""
                Verifys the connection to the database
                and checks read/write access
                """)
@click.pass_context
def ping(ctx):
    """
    Verifys the connection to the database
    and checks read/write access
    :docid: string (will be converted to bson object)
    :returns: bool
    """
    utils.log_info("Trying to connect to database...")
    response = db.db_ping(ctx)
    if response:
        utils.log_info("Connection SUCCESSFUL")
    else:
        utils.log_error("Connection FAILED")

    response = db.db_verify(ctx)
    utils.log_info("Trying to write, read, delete to test_collection...")
    if response:
        utils.log_info("Interactions were SUCCESSFUL")
    else:
        utils.log_error("Interactions FAILED")

    if not response:
        print("")
        utils.log_error("Please check your configuration file")
        utils.log_error("You may have:")
        utils.log_error("* a syntax error")
        utils.log_error("* the connection scheme is wrong")
        utils.log_error("* the mongodb instance is not running")
        utils.log_error("* the mongodb instance is not reachable")
        utils.log_error("* the authentication on mongodb side does not work or is not enabled")
        return False
    else:
        print("")
        utils.log_info("Awesome, you can go ahead and use rvo!")

    return True



import datetime
import rvo.db as db

def log(ctx, document, type, title):
    """
    Logs any interaction with rvo to the database
    One action generates exactly one entry in the transactions
    collection. The purpose is to be able to track changes
    and see what happens.
    :document: str (objectid)
    :type: str (show, mail, modify, info, add, edit)
    :message:
    :returns: bool
    """
    d = datetime.datetime.now()
    item = {
        "document": document,
        "date": d,
        "type": type,
        "title": title
    }
    db.add_transaction(ctx, item)




import sys
import click
from bson import ObjectId
import rvo.db as db
import rvo.utils as utils
import rvo.transaction as transaction
from rvo.crypto import crypto
from rvo.validate import validate
import rvo.config

@click.command(short_help="Shows a document",
              help="""
              Shows a single document from documentstore
              and opens its content in a pager (or stdout).
              """)
@click.argument('docid')
@click.option('password', '-p', '--password', required=False, default=False,
              help="Password for encrypted documents")
@click.option('stdout', '-s', '--stdout', required=False, default=False, is_flag=True,
              help="Print document to stdout instead of opening pager")
@click.pass_context
def show(ctx, docid, password, stdout):
    """
    Shows a single object from database
    and opens its content in a pager
    :docid: string (will be converted to bson object)
    :returns: bool
    """

    coll = db.get_document_collection(ctx)
    doc, docid = db.get_document_by_id(ctx, docid)

    config = ctx.obj["config"]

    content, c = db.get_content(ctx, doc, password=password)

    if sys.stdout.isatty() and not stdout:
        utils.view_content_in_pager(config["pager"], config['pageropts'], template=content)
    else:
        # try:
        #     print(content.encode("utf-8"))
        # except UnicodeDecodeError:
        print(content.encode("utf-8"))

    transaction.log(ctx, docid, "show", doc["title"])

    # showing the message means decrypting the message
    # in order to do not reuse the nonce from salsa20,
    # we have to reencrypt the content and update the field
    if doc["encrypted"] is True:
        doc["content"] = c.encrypt_content(content.encode("utf-8"))
        if validate(doc):
            coll.save(doc)
        else:
            utils.log_error("Validation of the updated object did not succeed")


    return True




import click
import rvo.db as db
import rvo.utils as utils
import rvo.transaction as transaction
import rvo.config
import simplemail


@click.command(short_help="Mails a document to a recipient for your choice",
               help="""
    Mails a document to a recipient for your choice.
    Its required to have a local unauthenticated smtpd running
    otherwise sending mails will fail.

    Im so s0rr3y.
    """)
@click.option('to', '-t', '--to', type=str, prompt="%s To" % utils.query_prefix,
              help='Recipient of the mail')
@click.argument('docid')
@click.pass_context
def mail(ctx, docid, to):
    """
    Mails a document to a recipient for your choice
    Its required to have a local unauthenticated smtpd running
    otherwise sending mails will fail
    :docid: str (Bson Object)
    :returns: bool
    """

    coll = db.get_document_collection(ctx)
    config = ctx.obj["config"]
    doc, docid = db.get_document_by_id(ctx, docid)

    try:
        simplemail.Email(
            smtp_server = "localhost",
            from_address = config["mailfrom"],
            to_address = to,
            subject = unicode(doc["title"]),
            message = doc["content"].encode("utf-8")
        ).send()
    except:
        utils.log_error("Error trying to send mail. May check your configuration.")



import sys
import re
import click
import fileinput
import datetime
import rvo.config
import rvo.db as db
import rvo.views as views
import rvo.utils as utils
import rvo.transaction as transaction
from rvo.validate import validate, validate_date, validate_location
from rvo.crypto import crypto

@click.command(help="""
            Add a document. 'add' is made to be very intuitive and functional.
            You can add and set all fields of a document during creation.

            It also supports reading from stdin. Unless this isn't the case
            it will start up your configured editor.
            """)
@click.option('encrypt', '-e' ,'--encrypted', default=False, is_flag=True, help='Encrypt this document')
@click.option('tags', '-t' ,'--tag',
              type=str, required=False, show_default=False, multiple=True,
              help='Set tags for this document')
@click.option('categories', '-c' ,'--category',
              type=str, required=False, show_default=False, multiple=True,
              help='Set categories for this document')
@click.option('date', '-d', '--date', default=datetime.datetime.now(),
              help='Set a custom creation date for this document', callback=validate_date)
@click.option('location', '-l', '--location', default=None, required=False,
              help='Set a location for this document', callback=validate_location)
@click.option('password', '-p', '--password', required=False, default=False,
              help="Password for encrypted documents")
@click.option('content', '-x', '--content', default=False, required=False, help='Read content from parameter')
@click.pass_context
def add(ctx, date, tags, categories, location, content, password, encrypt):
    """
    Adds a new document with various meta data options.
    """

    config = ctx.obj["config"]

    # Kindly ask for password when encryption comes
    if encrypt:
        c = crypto(ctx, password)
        if not c:
            sys.exit(1)

    # If input from -x parameter, encode in utf8
    try:
        content = content.encode("utf8")
    except (AttributeError, UnicodeEncodeError):
        pass

    # Read content directly from stdin if:
    # * there is no content from -x
    # * document should not be encrypted (conflicts with password prompt)
    # * there is piped input
    if not content and not encrypt and not utils.isatty:
        content = ""
        for l in click.get_text_stream('stdin'):
                content = content + l
        if content == "":
            content = False

    # If there is still no content and there is an interactive terminal,
    # open the editor to get query
    if not content and utils.isatty:
        content, tmpfile = utils.get_content_from_editor(config["editor"])

    # If everything fails, fail silently and let the user know.
    if not content:
        utils.log_error("Could not get any content")
        sys.exit(1)

    # Generate title from content
    title = utils.get_title_from_content(content)

    # Remove emojis
    title = utils.remove_emojis(title)

    # Encrypt
    if encrypt is True:
        content = c.encrypt_content(content)

    # if content looks like a url, set category and fetch title
    if re.compile("^https?://").search(content.lower()) is not None:

        # remove whitespace from right side
        content = content.rstrip()

        # check for duplicates
        if db.check_for_duplicate(ctx, field="url", content=content) is True:
            utils.log_info("Duplicate found")

        # fetch title and set category
        title = utils.get_title_from_webpage(content)

        categories = ["links"]

    # location
    loc = {}
    if location is None:
        loc["address"] = None
        loc["lat"] = None
        loc["long"] = None
    else:
        loc["address"] = location.address
        loc["lat"] = location.latitude
        loc["long"] = location.longitude

    # build item to insert into collection
    item = {
        "title": title,
        "content": content,
        "tags": list(tags),
        "categories": list(categories),
        "created": date,
        "updated": date,
        "encrypted": encrypt,
        "location": loc,
    }

    # insert item if its valid
    if validate(item):
        coll = db.get_document_collection(ctx)
        docid = coll.insert_one(item).inserted_id

        transaction.log(ctx, str(docid), "add", title)
        utils.log_info("Document \"%s\" created." % title)

    else:
        utils.log_error("Validation of the updated object did not succeed")

    try:
        utils.clean_tmpfile(tmpfile)
    except UnboundLocalError:
        pass

    return True



import re
import click
import datetime
import rvo.db as db
import rvo.utils as utils
import rvo.views as views
from rvo.validate import validate_date

@click.command(short_help="List documents from store",
               help="""
               Lists documents from the documentstore

               By default, the 10 latest documents will be shown.
               As documents grow, it gets difficult to find those.

               Filters can be applied on list command.
               See options below on how to filter the output.

               """)
@click.option('-t', '--tag', type=str, multiple=True,
              help='Results have to contain this tag')
@click.option('-c', '--category', type=str, multiple=True,
              help='Results need to be in this category')
@click.option('-s', '--title', type=str, multiple=True,
              help='Results need to contain this string in title')
@click.option('-x', '--content', type=str, multiple=True,
              help='Results need to contain this string in content')
@click.option('-l', '--limit', type=int, default=10,
              help='Limit the number of results')
@click.option('datefrom', '-f', '--from', default=datetime.datetime.utcfromtimestamp(0),
              callback=validate_date,
              help='Results by date starting from')
@click.option('dateto', '-d', '--to', default=datetime.datetime.now(),
              callback=validate_date,
              help='Results by date ending at date')
@click.option('-o', '--order', type=click.Choice(['created', 'updated']), default="updated",
              help='Specify sorting of the results')
@click.pass_context
def list(ctx, tag, category, title, content, limit, dateto, datefrom, order):
    """
    Lists documents from database based on the filters
    given. First stage is triggering the filter parser,
    building the query and finally printing results by
    calling the view.
    :filters: string (formatted as l:foo or x:bar)
    :returns: bool
    """
    coll = db.get_document_collection(ctx)

    tags = utils.normalize_element(tag, "tags")
    categories = utils.normalize_element(category, "categories")
    title = utils.normalize_element(title, "title")
    content = utils.normalize_element(content, "content")

    df = { "updated": { "$gte": datefrom }}
    de = { "updated": { "$lte": dateto }}

    query = {"$and": [ tags, categories, content, title, de, df ] }

    print("")
    documents = {}
    c = 0

    # Fetch results from collection and feed it into a numbered
    # dictonary
    for doc in coll.find(query).sort(order, -1).limit(limit):
        c += 1
        documents[c] = doc

    # If there are less entries in collection
    # then the limit is, overwrite limit with c
    if c < limit:
        limit = c

    db.clean_shortids(ctx)
    # reverse the order using the numbered keys
    # and display the object
    for x in reversed(range(1, limit+1)):
        db.map_shortid(ctx, sid=x, oid=documents[x]["_id"])
        documents[x]["sid"] = x

    views.table(documents, limit+1)

    # print summary of results
    results = coll.find(query).count()
    utils.log_info("%s out of %s result(s)." % (len(documents), results))

    return True




import datetime
import click
from bson import ObjectId
import rvo.db as db
import rvo.utils as utils
import rvo.transaction as transaction
from rvo.crypto import crypto
from rvo.validate import validate_date

@click.command(short_help="Modifies a documents metadata",
               help="""
               Modifies a documents meta data.
               See options for what you can modify.
               """)
@click.argument('docid')
@click.option('encrypt', '-e', '--encrypted', type=click.Choice(['yes', 'no', 'unchanged']),
              default="unchanged", help='Encrypt this document')
@click.option('tags', '-t', '--tag',
              type=str,
              multiple=True,
              required=False,
              show_default=False,
              help='Set tags for this document')
@click.option('categories', '-c', '--category',
              type=str,
              multiple=True,
              required=False,
              help='Set categories for this document')
@click.option('date', '-d', '--date', default=None,
              help='Set a custom creation date for document', callback=validate_date)
@click.pass_context
def modify(ctx, docid, tags, categories, encrypt, date):
    """
    Modifies a documents meta data
    :docid: str (objectid)
    :tags: list
    :categories: list
    :encrypt: str
    :returns: bool
    """
    coll = db.get_document_collection(ctx)
    d = datetime.datetime.now()

    doc, docid = db.get_document_by_id(ctx, docid)

    # this is a total crazy hack to get
    # tuple ("a", "b", "c", " ", "d", "e")
    # into
    # list ['abc', 'de']
    # tags = ''.join(list(tags)).split()
    # categories = ''.join(list(categories)).split()
    tags = list(tags)
    categories = list(categories)

    if len(tags) > 0:
        coll.update({"_id": ObjectId(docid)}, {"$set": {"tags": tags, "updated": d}})
        utils.log_info("Updated tags to %s" % ', '.join(tags))
        transaction.log(ctx, docid, "tags", doc["title"])

    if len(categories) > 0:
        coll.update({"_id": ObjectId(docid)}, {"$set": {"categories": categories, "updated": d}})
        utils.log_info("Updated categories to %s" % ', '.join(categories))
        transaction.log(ctx, docid, "category", doc["title"])

    if encrypt == "yes":
        if doc["encrypted"] is False:
            c = crypto(ctx=ctx, password=False)
            content = c.encrypt_content(doc["content"].encode("utf-8"))
            coll.update({"_id": ObjectId(docid)}, {"$set": {"content": content, "encrypted": True}})
            utils.log_info("Document %s is now stored encrypted" % doc["title"])
            transaction.log(ctx, docid, "encrypted", doc["title"])
        else:
            utils.log_error("Document %s is already stored encrypted" % doc["title"])

    if encrypt == "no":
        if doc["encrypted"] is True:
            c = crypto(ctx=ctx, password=False)
            content = c.decrypt_content(doc["content"])
            coll.update({"_id": ObjectId(docid)}, {"$set": {"content": content, "encrypted": False}})
            utils.log_info("Document %s is now stored in plaintext" % doc["title"])
            transaction.log(ctx, docid, "decrypted", doc["title"])
        else:
            utils.log_error("Document %s is already stored in plaintext" % doc["title"])

    if date != None:
        coll.update({"_id": ObjectId(docid)}, {"$set": {"created": date, "updated": d}})
        utils.log_info("Updated creation date to %s" % date)
        transaction.log(ctx, docid, "date", doc["title"])

    return True



import click
import datetime
from bson import ObjectId
import rvo.db as db
import rvo.utils as utils
import rvo.transaction as transaction
from rvo.crypto import crypto
from rvo.validate import validate

@click.command(help="""
               Append content to a document.
               This is useful for programmatically adding content.
               """)
@click.option('password', '-p', '--password', required=False, default=False,
              help="Password for encrypted documents")
@click.option('content', '-x', '--content', required=False, default=False,
              help="Content to be appended to the document")
@click.argument('docid')
@click.pass_context
def append(ctx, docid, password, content):
    """
    Append string to a document

    Trys to find the object, (decrypt,) adds content to the bottom,
    (encrypt,) update date fields and regenerates the title.

    :docid: str (bson object)
    :returns: bool
    """

    coll = db.get_document_collection(ctx)
    doc, docid = db.get_document_by_id(ctx, docid)

    template, c = db.get_content(ctx, doc, password=password)

    d = datetime.datetime.now()

    if not content:
        content = ""
        for l in click.get_text_stream('stdin'):
                content = content + l

    content = template + content

    if isinstance(content, unicode):
        content = content.encode("utf-8")

    if doc["encrypted"] is True:
        title = utils.get_title_from_content(content)
        content = c.encrypt_content(content)
    else:
        if not "links" in doc["categories"]:
            title = utils.get_title_from_content(content)


    if content != template:
        doc["content"] = content
        doc["title"] = title
        doc["updated"] = d
        if validate(doc):
            coll.save(doc)
            transaction.log(ctx, docid, "append", title)
            utils.log_info("Content appended to \"%s\"." % title)
        else:
            utils.log_error("Validation of the updated object did not succeed")
    else:
        utils.log_info("No changes detected for \"%s\"" % title)

    return True



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
import os
import sys

# file read helper
def read_from_file(path):
    if os.path.exists(path):
        with open(path,"rb","utf-8") as input:
            return input.read()

version = "23.0.3"

setup(
    name='rvo',
    version=version,
    description='Managing text data from the commandline',
    long_description=read_from_file('README.rst'),
    url='https://github.com/noqqe/rvo',
    author='Florian Baumann',
    author_email='flo@noqqe.de',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Operating System :: POSIX :: BSD :: OpenBSD',
        'Topic :: Terminals',
        'Topic :: Utilities',
        'Topic :: Documentation',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
    ],
    keywords='links mongodb quotes notes journal diary',
    packages=find_packages(),
    zip_safe=True,
    install_requires=['pymongo', 'configparser', 'BeautifulSoup4',
                      'pynacl', 'pyblake2', 'tabulate', 'click',
                      'python-dateutil', 'python-simplemail', 'nltk',
                      'hurry.filesize', 'geopy'],
    entry_points={
        'console_scripts': [
            'rvo=rvo.cli:cli',
        ],
    },
)



import click
import rvo.db as db
import rvo.transaction as transaction
import rvo.views as views
import rvo.analysis as analysis

@click.command(short_help="Shows metadata of a document",
               help="""
               Shows metadata of a document.

               Metadata matters! NSA kills because of metadata.
               """)
@click.argument('docid')
@click.pass_context
def info(ctx, docid):
    """
    Shows all available meta data belonging to a signel document
    :docid: string (will be converted to bson object)
    :returns: bool
    """
    doc, docid = db.get_document_by_id(ctx, docid)
    transaction.log(ctx, docid, "info", doc["title"])
    views.detail(doc)
    return True



import click
import datetime
from bson import ObjectId
import rvo.db as db
import rvo.utils as utils
import rvo.transaction as transaction
from rvo.crypto import crypto
from rvo.validate import validate
import rvo.config

@click.command(help="""
               Edit the content of an document.

               Your configured editor will be opened and you
               are free to type any content you like.
               Once you save and close the editor, content will be
               read and added to the store.

               If the content does not get changed, rvo detects
               there was no change and does not update.
               """)
@click.argument('docid')
@click.option('password', '-p', '--password', required=False, default=False,
              help="Password for encrypted documents")
@click.pass_context
def edit(ctx, docid, password):
    """
    Edit the content of an document.
    Trys to find the object, (decrypt,) read from editor,
    encrypt, update date fields and regenerates the title.
    :docid: str (bson object)
    :returns: bool
    """
    coll = db.get_document_collection(ctx)
    config = ctx.obj["config"]

    doc, docid = db.get_document_by_id(ctx, docid)
    title = doc["title"]

    template, c = db.get_content(ctx, doc, password=password)

    content, tmpfile = utils.get_content_from_editor(config["editor"], template=template)
    d = datetime.datetime.now()

    if doc["encrypted"] is True:
        title = utils.get_title_from_content(content)
        content = c.encrypt_content(content.decode("utf-8").encode("utf-8"))
    else:
        if not "links" in doc["categories"]:
            title = utils.get_title_from_content(content)

    if isinstance(template, unicode):
        content = content.decode("utf-8")

    if content != template:
        doc["content"] = content
        doc["title"] = title
        doc["updated"] = d
        if validate(doc):
            coll.save(doc)
        else:
            utils.log_error("Validation of the updated object did not succeed")

        transaction.log(ctx, docid, "edit", title)
        utils.log_info("Document \"%s\" updated." % title)
    else:
        utils.log_info("No changes detected for \"%s\"" % title)

    utils.clean_tmpfile(tmpfile)

    return True
